@@ -18,7 +18,9 @@
 
 /* getent: get entries from administrative database.  */
 
-//#include <aliases.h>
+#ifndef __CYGWIN__
+#include <aliases.h>
+#endif
 #include <argp.h>
 #include <ctype.h>
 #include <error.h>
@@ -48,26 +50,18 @@
 #include <netinet/in.h>
 #include <sys/socket.h>
 
-#ifndef __CYGWIN__
 /* Get libc version number.  */
 #include <version.h>
-#else
-#include <windows.h>
-#include <wchar.h>
-#include <sddl.h>
-#include <sys/cygwin.h>
+
+#ifdef __CYGWIN__
+#include "cyghelper.h"
 
 #define _(string) (string)
 #define N_(string) (string)
 #define textdomain(string)
 #define gettext(string) (string)
 
-#define fputc_unlocked fputc
-#define fputs_unlocked fputs
-#define putchar_unlocked putchar
-
 #define PKGVERSION	"(Cygwin)"
-#define VERSION		"2.18.90"	/* glibc version we took getent from. */
 #define REPORT_BUGS_TO	"http://cygwin.com/problems.html"
 
 # define stackinfo_alloca_round(l) (((l) + 15) & -16)
@@ -121,7 +115,7 @@ static struct argp argp =
 static int idn_flags = AI_IDN | AI_CANONIDN;
 
 #ifdef __CYGWIN__
-static int windows_only = 0;
+int windows_only = 0;
 #endif
 
 /* Print the version information.  */
@@ -228,107 +222,12 @@ ethers_keys (int number, char *key[])
 }
 #endif /* !__CYGWIN__ */
 
-#ifdef __CYGWIN__
-static PSID
-get_sid_from_special_string (const char *key, PBOOL try_as_cygname)
-{
-  PSID sid = NULL;
-
-  if (!strncmp (key, "U-", 2))
-    {
-      WCHAR wnam[4096];
-
-      /* DNLEN is moot.  Domain names can now be twice as long. */
-      WCHAR dom[32];
-      DWORD dlen = 32;
-      DWORD slen = SECURITY_MAX_SID_SIZE;
-      SID_NAME_USE use;
-
-      memset (wnam, 0, sizeof wnam);
-      mbstowcs (wnam, key + 2, sizeof wnam / sizeof (wchar_t) - 1);
-      sid = (PSID) LocalAlloc (LMEM_FIXED, SECURITY_MAX_SID_SIZE);
-      if (sid && !LookupAccountNameW (NULL, wnam, sid, &slen,
-				      dom, &dlen, &use))
-	{
-	  LocalFree (sid);
-	  sid = NULL;
-	}
-      if (sid
-	  && (use == SidTypeDomain || use == SidTypeComputer)
-	  && wcslen (wnam) < sizeof wnam / sizeof (wchar_t) / 2 - 2)
-	{
-	  wcpcpy (wnam + wcslen (wnam) + 1, wnam);
-	  wnam[wcslen (wnam)] = L'\\';
-	  dlen = 32;
-	  slen = SECURITY_MAX_SID_SIZE;
-	  if (!LookupAccountNameW (NULL, wnam, sid, &slen,
-				   dom, &dlen, &use))
-	    {
-	      LocalFree (sid);
-	      sid = NULL;
-	    }
-	}
-    }
-  else if (!strncmp (key, "S-1-", 4))
-    {
-      *try_as_cygname = TRUE;
-      ConvertStringSidToSidA (key, &sid);
-    }
-  return sid;
-}
-
-static void
-print_windows_mapping (const char *cygname, unsigned long id,
-		       const char *sid_string)
-{
-  PSID sid;
-  WCHAR name[NAME_MAX + 1];
-  DWORD nlen = NAME_MAX + 1;
-  /* DNLEN is moot.  Domain names can now be twice as long. */
-  WCHAR dom[32];
-  DWORD dlen = 32;
-  SID_NAME_USE use;
-
-  if (!cygname || !sid_string)
-    return;
-  if (!ConvertStringSidToSidA (sid_string, &sid))
-    return;
-  if (LookupAccountSidW (NULL, sid, name, &nlen, dom, &dlen, &use))
-    printf ("%s:%lu:%ls\\%ls:%s\n", cygname, id, dom, name, sid_string);
-  LocalFree (sid);
-}
-#endif
-
 /* This is for group */
 static void
 print_group (struct group *grp)
 {
-  unsigned int i = 0;
-
-#ifdef __CYGWIN__
-  /* Disable printing gid -1, the fallback invalid group called "????????" on
-     Cygwin versions prior to the introduction of direct SAM/AD access. */
-  if (grp->gr_gid == (gid_t) -1)
-    return;
-  if (windows_only)
-    {
-      print_windows_mapping (grp->gr_name, (unsigned long) grp->gr_gid,
-			     grp->gr_passwd);
-      return;
-    }
-#endif
-  printf ("%s:%s:%lu:", grp->gr_name ? grp->gr_name : "",
-	  grp->gr_passwd ? grp->gr_passwd : "",
-	  (unsigned long int) grp->gr_gid);
-
-  while (grp->gr_mem[i] != NULL)
-    {
-      fputs_unlocked (grp->gr_mem[i], stdout);
-      ++i;
-      if (grp->gr_mem[i] != NULL)
-	putchar_unlocked (',');
-    }
-  putchar_unlocked ('\n');
+  if (putgrent (grp, stdout) != 0)
+    fprintf (stderr, "error writing group entry: %m\n");
 }
 
 static int
@@ -358,25 +257,11 @@ group_keys (int number, char *key[])
 	grp = getgrgid (arg_gid);
 #ifdef __CYGWIN__
       /* Allow searching for Windows SIDs. */
-      else
-	{
-	  BOOL try_as_cygname = FALSE;
-	  PSID sid = get_sid_from_special_string (key[i], &try_as_cygname);
-
-	  if (sid)
-	    {
-	      grp = (struct group *) cygwin_internal (CW_GETGRSID, 0, sid);
-	      if (grp == (struct group *) -1) /* pre 1.7.29 Cygwin */
-		grp = NULL;
-	      LocalFree (sid);
-	    }
-	  else if (!try_as_cygname)
-	    grp = getgrnam (key[i]);
-	}
+      else if (!(grp = getgrsid (key[i])))
 #else
       else
-	grp = getgrnam (key[i]);
 #endif
+	grp = getgrnam (key[i]);
 
       if (grp == NULL)
 	result = 2;
@@ -769,26 +654,8 @@ networks_keys (int number, char *key[])
 static void
 print_passwd (struct passwd *pwd)
 {
-#ifdef __CYGWIN__
-  /* Disable printing uid -1, the fallback invalid user called "????????" on
-     Cygwin versions prior to the introduction of direct SAM/AD access. */
-  if (pwd->pw_uid == (uid_t) -1)
-    return;
-  if (windows_only)
-    {
-      char *c = strstr (pwd->pw_gecos, "S-1-");
-      print_windows_mapping (pwd->pw_name, (unsigned long) pwd->pw_uid, c);
-      return;
-    }
-#endif
-  printf ("%s:%s:%lu:%lu:%s:%s:%s\n",
-	  pwd->pw_name ? pwd->pw_name : "",
-	  pwd->pw_passwd ? pwd->pw_passwd : "",
-	  (unsigned long int) pwd->pw_uid,
-	  (unsigned long int) pwd->pw_gid,
-	  pwd->pw_gecos ? pwd->pw_gecos : "",
-	  pwd->pw_dir ? pwd->pw_dir : "",
-	  pwd->pw_shell ? pwd->pw_shell : "");
+  if (putpwent (pwd, stdout) != 0)
+    fprintf (stderr, "error writing passwd entry: %m\n");
 }
 
 static int
@@ -818,25 +685,11 @@ passwd_keys (int number, char *key[])
 	pwd = getpwuid (arg_uid);
 #ifdef __CYGWIN__
       /* Allow searching for Windows SIDs. */
-      else
-	{
-	  BOOL try_as_cygname = FALSE;
-	  PSID sid = get_sid_from_special_string (key[i], &try_as_cygname);
-
-	  if (sid)
-	    {
-	      pwd = (struct passwd *) cygwin_internal (CW_GETPWSID, 0, sid);
-	      if (pwd == (struct passwd *) -1) /* pre 1.7.29 Cygwin */
-		pwd = NULL;
-	      LocalFree (sid);
-	    }
-	  else if (!try_as_cygname)
-	    pwd = getpwnam (key[i]);
-	}
+      else if (!(pwd = getpwsid (key[i])))
 #else
       else
-	pwd = getpwnam (key[i]);
 #endif
+	pwd = getpwnam (key[i]);
 
       if (pwd == NULL)
 	result = 2;
